<?php
/*
    pmacct-fe is Copyright (C) 2005 by Paolo Lucente

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

@define('PFE_BASE', dirname(__FILE__));
@define('PFE_TEMPLATES', dirname(__FILE__) . '/templates');
@define('HORDE_BASE', dirname(__FILE__) . '/..');
require_once HORDE_BASE . '/lib/base.php';

/* User has not logged in before; just go to the login page. */
if (!Auth::isAuthenticated()) {
    Horde::authenticationFailureRedirect();
}

require_once PFE_BASE . '/SQLfuncs.php';
require_once PFE_BASE . '/Layoutfuncs.php';
require_once PFE_BASE . '/Miscfuncs.php';
require_once PFE_BASE . '/Util.php';
require_once PFE_BASE . '/Validate.php';

if (!CheckConfig()) {
    require PFE_TEMPLATES . '/common-header.inc';
    PrintMissingConfig();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

if (!Util::getFormData('obspoint') || !Util::getFormData('obj') || !Util::getFormData('obj2')
    || !Util::getFormData('pstart') || !Util::getFormData('pend') || !Util::getFormData('reptype')) {
    require PFE_TEMPLATES . '/common-header.inc';
    HandleReportFormError();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

/* assumed that we have all parameters, let's do some brief check */
$fail = FALSE;
if (!is_numeric(Util::getFormData('reptype'))) $fail = TRUE;
if (!is_numeric(Util::getFormData('numpoints'))) $fail = TRUE;
if (ValidateDateFormat(urldecode(Util::getFormData('pstart'))) < 0) $fail = TRUE;
if (ValidateDateFormat(urldecode(Util::getFormData('pend'))) < 0) $fail = TRUE;
if (ValidateStringLight(urldecode(Util::getFormData('obj2'))) < 0) $fail = TRUE;
if (ValidateObsPoint(urldecode(Util::getFormData('obspoint'))) < 0) $fail = TRUE;
if (ValidateCapability(urldecode(Util::getFormData('obj')), Auth::getAuth(), urldecode(Util::getFormData('obspoint'))) < 0) $fail = TRUE;

if ($fail) {
    HandleReportFormError();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

/* we build the graph basing on the 'end'-period and the previous n points;
   this is because we don't need the 'start'-period. We need the period point
   breakdown instead, this is because we set to TRUE the 'select_si' value */ 
$Period = array();
$Period['start'] = FALSE; 
$Period['end'] = Util::getFormData('pend');
$Period['select_si'] = TRUE;

$SelectedObsPoint = SelectObsPoint(Util::getFormData('obspoint'));
if (!isset($SelectedObsPoint['id'])) {
    require PFE_TEMPLATES . '/common-header.inc';
    PrintBoxedNoReportEntries();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$SelectedReportType = SelectReportType(Util::GetFormData('reptype'));
if (!isset($SelectedReportType['id'])) {
    require PFE_TEMPLATES . '/common-header.inc';
    PrintNoReportForm();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

if (!Util::getFormData('numpoints')) {
    require PFE_TEMPLATES . '/common-header.inc';
    PrintPlotForm($_GET);
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$ReportTableEntries = array();
GetTrafficReportEntries($SelectedObsPoint, $SelectedReportType, Util::GetFormData('obj'), Util::GetFormData('obj2'), $Period,
			'stamp_inserted', Util::GetFormData('numpoints'), $ReportTableEntries);
if (!isset($ReportTableEntries[0])) {
    require PFE_TEMPLATES . '/common-header.inc';
    PrintBoxedNoReportEntries();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$object = Util::getFormData('obj2') ? Util::getFormData('obj2') : Util::getFormData('obj');
PlotEntries($SelectedReportType, urldecode($object), $ReportTableEntries, Util::GetFormData('numpoints'), $Period);

require PFE_TEMPLATES . '/footer.inc';

function PlotEntries(&$RepType, $object, &$Entries, $NumPoints, &$Period)
{
    $XLabelNum = 8; /* 500px (tot) / 62.5px (reserved) = 8 */
    $legend = "Total traffic for " . $object;
    $Points = array();
    $step = $RepType['report_prefs']['stepping'];

    include $_SESSION['config']['prefs']['jpgraph_path'] . "jpgraph.php";
    include $_SESSION['config']['prefs']['jpgraph_path'] . "jpgraph_line.php";

    $astep = SplitStep($step);
    CalcStep($astep);
    $Points[$NumPoints-1]['time'] = FindBasetime(strtotime($Period['end']), $astep);
    CalcPrevBasetimes($Points, $astep, $NumPoints);
    CalcPrevDates($Points, $NumPoints);
    MatchEntries($Points, $Entries, $NumPoints);

    $data = array();
    foreach ($Points as $index => $row) {
        $data[$index] = $row['tot_bytes']; 
	$Pattern = "/^[\d-]+ (\d+):(\d+):[\d]+$/i";
	$dates[$index] = preg_replace($Pattern, "\$1:\$2", $row['date']); 
    }

    $graph = new Graph(500,350,"auto"); /* It should fit in the Large popup window */
    $graph->SetScale("textint"); 
    $graph->img->SetMargin(100,10,10,35); 

    if ($XLabelNum >= $NumPoints) $XLabelInterval = $NumPoints; 
    else $XLabelInterval = bcdiv($NumPoints, $XLabelNum, 0); 
    $graph->xaxis->SetTextLabelInterval($XLabelInterval);
    $graph->SetShadow(); 
    $graph->yaxis->SetLabelFormatCallback("HandleBytes");
    $graph->yaxis->HideFirstLastLabel();
    $graph->xaxis->SetTickLabels($dates); 
    $graph->yaxis->SetTitle("bytes transferred", "middle"); 
    $graph->yaxis->SetTitleMargin(80); 
    $graph->xaxis->SetTitle("time (" . $step .  " average)", "middle"); 
    $graph->xaxis->SetTitleMargin(3); 
    $graph->ygrid->SetLineStyle('dotted');

    $lplot = new LinePlot($data); 
    $lplot->SetFillColor('#ffe697'); // Fill color 
    $lplot->SetColor('#ff9900'); // Line color 
    $lplot->SetLegend($legend); 
    $lplot->value->SetColor("black","navy"); 

    $graph->Add($lplot); 
    $graph->Stroke();
}

