<?php
/*
    pmacct-fe is Copyright (C) 2005 by Paolo Lucente

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

@define('PFE_BASE', dirname(__FILE__));
@define('PFE_TEMPLATES', dirname(__FILE__) . '/templates');
@define('HORDE_BASE', dirname(__FILE__) . '/..');
require_once HORDE_BASE . '/lib/base.php';

/* User has not logged in before; just go to the login page. */
if (!Auth::isAuthenticated()) {
    Horde::authenticationFailureRedirect();
}

require PFE_TEMPLATES . '/common-header.inc';
require_once PFE_BASE . '/SQLfuncs.php';
require_once PFE_BASE . '/Layoutfuncs.php';
require_once PFE_BASE . '/Miscfuncs.php';
require_once PFE_BASE . '/Util.php';
require_once PFE_BASE . '/Validate.php';

if (!CheckConfig()) {
    require PFE_TEMPLATES . '/common-header.inc';
    PrintMissingConfig();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

if (!Util::getFormData('obspoint') || !Util::getFormData('obj') || !Util::getFormData('dir')
    || !Util::getFormData('pstart') || !Util::getFormData('pend') || !Util::getFormData('reptype')
    || !Util::getFormData('obj2')) {
    HandleReportFormError();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

/* assumed that we have all parameters, let's do some brief check */
$fail = FALSE;
if (!is_numeric(Util::getFormData('reptype'))) $fail = TRUE;
if (Util::getFormData('dir') != 'in' && Util::getFormData('dir') != 'out' && Util::getFormData('dir') != 'both') $fail = TRUE;
if (ValidateDateFormat(urldecode(Util::getFormData('pstart'))) < 0) $fail = TRUE;
if (ValidateDateFormat(urldecode(Util::getFormData('pend'))) < 0) $fail = TRUE;
if (ValidateStringLight(urldecode(Util::getFormData('obj2'))) < 0) $fail = TRUE;
if (ValidateObsPoint(urldecode(Util::getFormData('obspoint'))) < 0) $fail = TRUE;
if (ValidateCapability(urldecode(Util::getFormData('obj')), Auth::getAuth(), urldecode(Util::getFormData('obspoint'))) < 0) $fail = TRUE;

if ($fail) {
    HandleReportFormError();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$Direction = Util::getFormData('dir');

$Period = array();
$Period['start'] = urldecode(Util::getFormData('pstart'));
$Period['end'] = urldecode(Util::getFormData('pend'));

$SelectedObsPoint = SelectObsPoint(Util::getFormData('obspoint'));
if (!isset($SelectedObsPoint['id'])) {
    PrintBoxedNoReportEntries();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$SelectedReportType = SelectReportType(Util::GetFormData('reptype'));
if (!isset($SelectedReportType['id'])) {
    PrintBoxedNoReportForm();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$PortSrcEntries = array();
$PortDstEntries = array();

if ($Direction == 'in' || $Direction == 'both')
    GetPortsEntries($SelectedObsPoint, $SelectedReportType, urldecode(Util::getFormData('obj')), Util::getFormData('obj2'),
		'in', $Period, $InPortSrcEntries, $InPortDstEntries); 

if ($Direction == 'out' || $Direction == 'both')
    GetPortsEntries($SelectedObsPoint, $SelectedReportType, urldecode(Util::getFormData('obj')), Util::getFormData('obj2'),
                'out', $Period, $OutPortSrcEntries, $OutPortDstEntries);

if (!isset($InPortSrcEntries[0]) && !isset($InPortDstEntries[0]) &&
    !isset($OutPortSrcEntries[0]) && !isset($OutPortDstEntries[0])) {
    PrintBoxedNoReportEntries();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

if ($Direction == 'in' || $Direction == 'both')
    PrintPortsReport($SelectedObsPoint, urldecode(Util::getFormData('obj2')), $SelectedReportType, 'in', $Period, 
		$InPortSrcEntries, $InPortDstEntries, TRUE);

if ($Direction == 'out' || $Direction == 'both')
    PrintPortsReport($SelectedObsPoint, urldecode(Util::getFormData('obj2')), $SelectedReportType, 'out', $Period,
                $OutPortSrcEntries, $OutPortDstEntries, FALSE);

require PFE_TEMPLATES . '/footer.inc';


