<?php
/*
    pmacct-fe is Copyright (C) 2005 by Paolo Lucente

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

function DB_Connect($Database)
{
    $Errmsg = FALSE;

    $SQLconnstring = BuildDBConnectionString($Database);
    ob_start(); /* Catch PostgreSQL errors */
    $dbconn = pg_connect($SQLconnstring) or $Errmsg = pg_last_error($dbconn); 
    ob_end_clean();
    if ($dbconn == FALSE) {
        PrintDBConnectionFailed($Errmsg);
        PrintReloadPageBox("Reload page");
        exit();
    }

    return $dbconn;
}

function DB_Query($Resource, $Query)
{
    $Errmsg = FALSE;

    ob_start(); /* Catch PostgreSQL errors */
    $res = pg_query($Resource, $Query) or $Errmsg = pg_last_error($Resource);
    ob_end_clean();
    if ($res == FALSE) {
        PrintDBQueryFailed($Errmsg);
	exit();
    }

    return $res;
} 

function DB_Close($Resource)
{
    pg_close($Resource);
}

function GetObsPoints(&$result)
{
    global $config;

    $dbconn = DB_Connect($config['pfe_DB']);

    $SQLquery = "SELECT * FROM observation_points";
    $res = DB_Query($dbconn, $SQLquery);

    for ($idx = 0; $row = pg_fetch_array($res); $idx++) $result[$idx] = $row;
    DB_Close($dbconn);
}

function GetReportDescriptions(&$result)
{
    global $config;

    $dbconn = DB_Connect($config['pfe_DB']);

    $SQLquery = "SELECT * FROM reports";
    $res = DB_Query($dbconn, $SQLquery);

    for ($idx = 0; $row = pg_fetch_array($res); $idx++) $result[$idx] = $row;

    foreach($result as $key => $row) {
	if (isset($row['prefs']) && (trim($row['prefs']) != 'none')) {
	    ob_start();
	    include PFE_BASE . "/" . trim($row['prefs']);
	    ob_end_clean();
	    if (isset($report_prefs)) $result[$key]['report_prefs'] = $report_prefs;
	    else {
		PrintFileNotFound(trim($row['prefs']));
		exit();
	    }
	}
    }

    DB_Close($dbconn);
}

function SelectObsPoint($RequestedObsPoint)
{
    GetCachedResults('config', $config);
    // global $config;
   
    $ObsPoints = array();
    GetCachedResults('obs_points', $ObsPoints);
    if (isset($ObsPoints[0])) { /* We got results */
        foreach($ObsPoints as $row) {
	    if (trim($row['id']) == $RequestedObsPoint) return $row;
	}
	return; /* No match */ 
    }
    else {
	$dbconn = DB_Connect($config['pfe_DB']);

        $SQLquery = "SELECT * FROM observation_points WHERE id = '" . $RequestedObsPoint . "'";
        $res = DB_Query($dbconn, $SQLquery);

        $row = pg_fetch_array($res);
        DB_Close($dbconn);

        return $row;
    }
}

function GetDates($ObsPoint, $table, &$result) 
{ 
    $dbconn = DB_Connect($ObsPoint);

    # $SQLquery = "SELECT DISTINCT(stamp_inserted) FROM " . $table . " ORDER BY stamp_inserted DESC";
    $SQLquery = "SELECT stamp_inserted FROM (SELECT stamp_inserted FROM " . $table . " GROUP BY stamp_inserted ) as acct_volatile ORDER BY stamp_inserted DESC";
    $res = DB_Query($dbconn, $SQLquery);

    for ($idx = 0; $row = pg_fetch_array($res); $idx++) $result[$idx] = $row;
    DB_Close($dbconn);
}

function GetCapabilities($user, $ObsPoint, &$result) 
{
    GetCachedResults('config', $config);
    // global $config;

    $dbconn = DB_Connect($config['pfe_DB']);

    $SQLquery = "SELECT * FROM capabilities WHERE username = '" . trim($user) . "' AND obs_point = '" . trim($ObsPoint['id']) . "'"; 
    $res = DB_Query($dbconn, $SQLquery);

    for ($idx = 0; $row = pg_fetch_array($res); $idx++) $result[$idx] = $row;
    DB_Close($dbconn);
}

function BuildDBConnectionString($DBparams)
{
    $SQLconnstring = "dbname=" . $DBparams['dbname'] .
              " user=" . $DBparams['username'] .
              " password=" . $DBparams['password'];
    if (isset($DBparams['hostname'])) {
        $SQLconnstring = $SQLconnstring . " host=" . $DBparams['hostname'];
    }

    return $SQLconnstring;
}

/*
 *
 * GetTrafficReportEntries() logics.
 *
 * a) all customizable fields are checked to understand whether to use default or user-supplied values; user values, if any, 
 *    should have been validated before. If any value causes an error with the DB, the related message is sent back to the
 *    user.
 * b) Ordering and limiting are evaluated: whether they are connected to the time period ('stamp_inserted'), they are handled
 *    via SQL queries; otherwise - e.g., total traffic ('tot_bytes') and network object ('ip_src') - they will be evaluated 
 *    via SortResults() and LimitResults() functions because the 'ordering values' come from multiple queries (their partial
 *    results need to be joined up before).
 * c) $Period is evaluated. It might exist or not; if it does, it's expected to contain two indexes: 'start' and 'end'; then
 *    the proper 'WHERE'-chunks are built. 
 * d) Grouping is evaluated. It may have been required explicitely by the user or is needed by the previous time evaluation.
 *    If a specific time period have been not requested, it's included in both the 'GROUP BY' and 'SELECT' chunks.
 * e) Queries are built; they are sent to the DB; results are loaded in vectors, ready to be post-processed.
 * f) Merge_In_Out_Results() joins up the results from multiple vectors. 
 * g) The resulting vector is sorted and limited if required. 
 *
 */
function GetTrafficReportEntries($ObsPoint, &$RepType, $obj, $obj2 = FALSE, &$Period, $order, $limit, &$result)
{
    $Defaults = $_SESSION['defaults'];

    if (isset($RepType['report_prefs']['enable_grouping']) && $RepType['report_prefs']['enable_grouping']) $Grouping = TRUE;
    else $Grouping = FALSE;

    /* Initializing values */
    $HaveObj = FALSE;
    $HaveObj2 = FALSE;
    $Values = array();
    $Limiting = FALSE;
    $Period_chunk = "";
    $Group_chunk = "";
    $Limit_chunk = "";
    $Order_chunk = "";
    $NoDirection = FALSE;

    /* Normalizing $Period values and more ... */
    if (!isset($Period['start'])) $Period['start'] = FALSE;
    if (!isset($Period['end'])) $Period['end'] = FALSE;
    if (!isset($Period['select_si'])) $Period['select_si'] = FALSE;
    if (isset($RepType['report_prefs']['no_direction'])) unset($RepType['report_prefs']['no_direction']);

    $Values['table_in'] = isset($RepType['report_prefs']['table_in']) ? $RepType['report_prefs']['table_in'] : $Defaults['table_in']; 
    $Values['table_out'] = isset($RepType['report_prefs']['table_out']) ? $RepType['report_prefs']['table_out'] : $Defaults['table_out']; 
    $Values['qfield_in'] = isset($RepType['report_prefs']['qfield_in']) ? $RepType['report_prefs']['qfield_in'] : $Defaults['qfield_in']; 
    $Values['qfield_out'] = isset($RepType['report_prefs']['qfield_out']) ? $RepType['report_prefs']['qfield_out'] : $Defaults['qfield_out'];
    $Values['sfield_in'] = isset($RepType['report_prefs']['sfield_in']) ? $RepType['report_prefs']['sfield_in'] : $Defaults['sfield_in'];
    $Values['sfield_out'] = isset($RepType['report_prefs']['sfield_out']) ? $RepType['report_prefs']['sfield_out'] : $Defaults['sfield_out'];

    if ($Values['table_in'] == $Values['table_out'] &&
        $Values['qfield_in'] == $Values['qfield_out'] &&
	$Values['sfield_in'] == $Values['sfield_out']) {
	$RepType['report_prefs']['no_direction'] = TRUE; /* sticks the flag for later uses */
	$NoDirection = TRUE; 
    }

    if ($obj && strtolower($obj) != 'all') $HaveObj = TRUE; 
    if ($obj2) $HaveObj2 = TRUE; 

    if ($order == 'stamp_inserted') {
        $Order_chunk = " ORDER BY " . $order . " DESC";
	if ($limit) $Limit_chunk = " LIMIT " . $limit; /* Limiting, 1st stage */
	$Limiting = TRUE;
    }

    /* Handling time period */
    if (getPeriod($Period)) {
        if (!$HaveObj && !$HaveObj2) $Period_chunk = " WHERE ";
        else $Period_chunk = " AND ";

        if ($Period['start'] == $Period['end']) $Period_chunk .= " stamp_inserted = '" . $Period['start'] . "'"; 
        else {
            if ($Period['start']) {
                $Period_chunk .= " stamp_inserted >= '" . $Period['start'] . "'";
                if ($Period['end']) $Period_chunk .= " AND ";
            }
            if ($Period['end']) $Period_chunk .= " stamp_inserted <= '" . $Period['end'] . "'";
            $Grouping = TRUE;
        }
    }

    /* $Select_chunk is chained to the specific sfield */
    if ($Grouping) {
	if (!getPeriodSI($Period)) {
	    $Select_chunk = ", SUM(packets) as packets, SUM(bytes) as bytes, MAX(stamp_updated) as stamp_updated ";
	}
	else {
	    $Select_chunk = ", stamp_inserted, SUM(packets) as packets, SUM(bytes) as bytes, MAX(stamp_updated) as stamp_updated ";
	    $Group_chunk = ", stamp_inserted";
	}
    }
    else {
	if (!getPeriodSI($Period)) {
	    $Select_chunk = ", packets, bytes, stamp_updated ";
	}
	else $Select_chunk = ", packets, bytes, stamp_inserted, stamp_updated ";
    }

    $dbconn = DB_Connect($ObsPoint);

    /* The following queries are not optimized for the case qfield == sfield
       the purpose is to ensure that $obj2 is dependant from $obj. Operating
       this way, we avoid trivial $obj2 mangling issues */
    if (strstr($obj, '/')) { /* network prefix */
        $SQLquery_in = "SELECT " . $Values['sfield_in'] . $Select_chunk;
	$SQLquery_in .= " FROM " . $Values['table_in']; 
        if ($HaveObj || $HaveObj2) $SQLquery_in .= " WHERE ";
        if ($HaveObj) $SQLquery_in .= $Values['qfield_in'] . " << '" . $obj . "'";
        if ($HaveObj && $HaveObj2) $SQLquery_in .= " AND ";
        if ($HaveObj2) $SQLquery_in .= $Values['sfield_in'] . " = '" . $obj2 . "'";
	$SQLquery_in .= $Period_chunk; 
	if ($Grouping) $SQLquery_in .= " GROUP BY " . $Values['sfield_in'] . $Group_chunk; 
	if ($Limiting) $SQLquery_in .= $Order_chunk . $Limit_chunk; 

	if (!$NoDirection) {
            $SQLquery_out = "SELECT " . $Values['sfield_out'] . $Select_chunk;
	    $SQLquery_out .= " FROM " . $Values['table_out'];
            if ($HaveObj || $HaveObj2) $SQLquery_out .= " WHERE ";
            if ($HaveObj) $SQLquery_out .= $Values['qfield_out'] . " << '" . $obj . "'";
            if ($HaveObj && $HaveObj2) $SQLquery_out .= " AND ";
            if ($HaveObj2) $SQLquery_out .= $Values['sfield_out'] . " = '" . $obj2 . "'";
	    $SQLquery_out .= $Period_chunk;
	    if ($Grouping) $SQLquery_out .= " GROUP BY " . $Values['sfield_out'] . $Group_chunk; 
	    if ($Limiting) $SQLquery_out .= $Order_chunk . $Limit_chunk; 
	}
    }
    else {
	$SQLquery_in = "SELECT " . $Values['sfield_in'] . $Select_chunk;
	$SQLquery_in .= " FROM " . $Values['table_in'];
        if ($HaveObj || $HaveObj2) $SQLquery_in .= " WHERE ";
        if ($HaveObj) $SQLquery_in .= $Values['qfield_in'] . " = '" . $obj . "'";
        if ($HaveObj && $HaveObj2) $SQLquery_in .= " AND ";
        if ($HaveObj2) $SQLquery_in .= $Values['sfield_in'] . " = '" . $obj2 . "'"; 
	$SQLquery_in .= $Period_chunk; 
	if ($Grouping) $SQLquery_in .= " GROUP BY " . $Values['sfield_in'] . $Group_chunk; 
	if ($Limiting) $SQLquery_in .= $Order_chunk . $Limit_chunk; 

	if (!$NoDirection) {
            $SQLquery_out = "SELECT " . $Values['sfield_out'] . $Select_chunk;
	    $SQLquery_out .= " FROM " . $Values['table_out'];
	    if ($HaveObj || $HaveObj2) $SQLquery_out .= " WHERE ";
	    if ($HaveObj) $SQLquery_out .= $Values['qfield_out'] . " = '" . $obj . "'"; 
	    if ($HaveObj && $HaveObj2) $SQLquery_out .= " AND ";
	    if ($HaveObj2) $SQLquery_out .= $Values['sfield_out'] . " = '" . $obj2 . "'";
	    $SQLquery_out .= $Period_chunk;
	    if ($Grouping) $SQLquery_out .= " GROUP BY " . $Values['sfield_out'] . $Group_chunk; 
	    if ($Limiting) $SQLquery_out .= $Order_chunk . $Limit_chunk; 
	}
    }

    $res_in = DB_Query($dbconn, $SQLquery_in);
    for ($idx = 0; $row = pg_fetch_array($res_in); $idx++) $Result_in[$idx] = $row;
    if (!$NoDirection) {
	$res_out = DB_Query($dbconn, $SQLquery_out);
	for ($idx = 0; $row = pg_fetch_array($res_out); $idx++) $Result_out[$idx] = $row;
    }

    Merge_In_Out_Results($Result_in, $Result_out, $Values, $Period, $result);
    SortResults($order, $result);
    if ($limit) LimitResults($limit, $result); 

    DB_Close($dbconn);
}

function GetPortsEntries($ObsPoint, $RepType, $obj, $obj2, $Direction, &$Period, &$SrcPorts, &$DstPorts) 
{
    $Defaults = $_SESSION['defaults'];

    $Values = array();
    $dbconn = DB_Connect($ObsPoint);

    $Values['qfield_in'] = isset($RepType['report_prefs']['qfield_in']) ? $RepType['report_prefs']['qfield_in'] : $Defaults['qfield_in'];
    $Values['qfield_out'] = isset($RepType['report_prefs']['qfield_out']) ? $RepType['report_prefs']['qfield_out'] : $Defaults['qfield_out'];
    $Values['sfield_in'] = isset($RepType['report_prefs']['sfield_in']) ? $RepType['report_prefs']['sfield_in'] : $Defaults['sfield_in'];
    $Values['sfield_out'] = isset($RepType['report_prefs']['sfield_out']) ? $RepType['report_prefs']['sfield_out'] : $Defaults['sfield_out'];

    if ($Direction == 'out') {
        $Values['table_ss'] = isset($RepType['report_prefs']['table_ss']) ? $RepType['report_prefs']['table_ss'] : $Defaults['table_ss'];
        $Values['table_sd'] = isset($RepType['report_prefs']['table_sd']) ? $RepType['report_prefs']['table_sd'] : $Defaults['table_sd'];
        $Values['qfield_ss'] = $Values['qfield_out'];
        $Values['qfield_sd'] = $Values['qfield_out'];
    }
    else if ($Direction == 'in') {
        $Values['table_ds'] = isset($RepType['report_prefs']['table_ds']) ? $RepType['report_prefs']['table_ds'] : $Defaults['table_ds'];
        $Values['table_dd'] = isset($RepType['report_prefs']['table_dd']) ? $RepType['report_prefs']['table_dd'] : $Defaults['table_dd'];
        $Values['qfield_ds'] = $Values['qfield_in'];
        $Values['qfield_dd'] = $Values['qfield_in'];
    }

    /* Handling time period */
    if ($Period['start'] == $Period['end']) {
        $Period_chunk = " AND stamp_inserted = '" . $Period['start'] . "'";
        $Select_chunk = ", bytes, stamp_updated"; /* chained to the specific sfield */
        $Grouping = FALSE;
    }
    else {
        $Period_chunk = " AND stamp_inserted >= '" . $Period['start'] . "'";
        $Period_chunk .= " AND stamp_inserted <= '" . $Period['end'] . "'";
        $Select_chunk = ", SUM(bytes) as bytes, MAX(stamp_updated) as stamp_updated"; /* chained to the specific sfield */
        $Grouping = TRUE;
    }
 
    if ($Direction == 'out') {
        $SQLquery_dst = "SELECT port_dst, ip_proto" . $Select_chunk;
	$SQLquery_dst .= " FROM " . $Values['table_sd'];
	$SQLquery_dst .= " WHERE " . $Values['sfield_out'] . " = '" . $obj2 . "'" . $Period_chunk;
        if (strstr($obj, '/')) $SQLquery_dst .= " AND " . $Values['qfield_sd'] . " << '" . $obj . "'";
	else if (strtolower($obj) == 'all');
        else $SQLquery_dst .= " AND " . $Values['qfield_sd'] . " = '" . $obj . "'";
	if ($Grouping) $SQLquery_dst .= " GROUP BY port_dst, ip_proto"; 
	$SQLquery_dst .= " ORDER BY bytes DESC";
	$result_dst = DB_Query($dbconn, $SQLquery_dst);

	$SQLquery_src = "SELECT port_src, ip_proto" . $Select_chunk;
	$SQLquery_src .= " FROM " . $Values['table_ss'];
	$SQLquery_src .= " WHERE " . $Values['sfield_out'] . " = '" . $obj2 . "'" . $Period_chunk;
        if (strstr($obj, '/')) $SQLquery_src .= " AND " . $Values['qfield_ss'] . " << '" . $obj . "'";
	else if (strtolower($obj) == 'all');
        else $SQLquery_src .= " AND " . $Values['qfield_ss'] . " = '" . $obj . "'";
	if ($Grouping) $SQLquery_src .= " GROUP BY port_src, ip_proto";
	$SQLquery_src .= " ORDER BY bytes DESC";
	$result_src = DB_Query($dbconn, $SQLquery_src);
    }
    else if ($Direction == 'in') {
	$SQLquery_dst = "SELECT port_dst, ip_proto" . $Select_chunk;
	$SQLquery_dst .= " FROM " . $Values['table_dd']; 
        $SQLquery_dst .= " WHERE " . $Values['sfield_in'] . " = '" . $obj2 . "'" . $Period_chunk;
        if (strstr($obj, '/')) $SQLquery_dst .= " AND " . $Values['qfield_dd'] . " << '" . $obj . "'";
	else if (strtolower($obj) == 'all');
        else $SQLquery_dst .= " AND " . $Values['qfield_dd'] . " = '" . $obj . "'";
	if ($Grouping) $SQLquery_dst .= " GROUP BY port_dst, ip_proto";
	$SQLquery_dst .= " ORDER BY bytes DESC";
	$result_dst = DB_Query($dbconn, $SQLquery_dst);

        $SQLquery_src = "SELECT port_src, ip_proto" . $Select_chunk;
        $SQLquery_src .= " FROM " . $Values['table_ds']; 
        $SQLquery_src .= " WHERE " . $Values['sfield_in'] . " = '" . $obj2 . "'" . $Period_chunk;
        if (strstr($obj, '/')) $SQLquery_src .= " AND " . $Values['qfield_ds'] . " << '" . $obj . "'";
	else if (strtolower($obj) == 'all');
        else $SQLquery_src .= " AND " . $Values['qfield_ds'] . " = '" . $obj . "'";
	if ($Grouping) $SQLquery_src .= " GROUP BY port_src, ip_proto";
        $SQLquery_src .= " ORDER BY bytes DESC"; 
	$result_src = DB_Query($dbconn, $SQLquery_src);
    }
    else { /* Direction value not supported */
	DB_Close($dbconn);
	return;
    }

    for ($idx = 0; $row = pg_fetch_array($result_src); $idx++) $SrcPorts[$idx] = $row;
    for ($idx = 0; $row = pg_fetch_array($result_dst); $idx++) $DstPorts[$idx] = $row;

    DB_Close($dbconn);
}

