<?php
/*
    pmacct-fe is Copyright (C) 2005 by Paolo Lucente

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

@define('PFE_BASE', dirname(__FILE__));
@define('PFE_TEMPLATES', dirname(__FILE__) . '/templates');
@define('HORDE_BASE', dirname(__FILE__) . '/..');
require_once HORDE_BASE . '/lib/base.php';

/* User has not logged in before; just go to the login page. */
if (!Auth::isAuthenticated()) {
    Horde::authenticationFailureRedirect();
}
else {
    $pfe_user = Auth::getAuth();
}

require PFE_TEMPLATES . '/common-header.inc';
require PFE_TEMPLATES . '/menu.inc';
$notification->notify(array('listeners' => 'status'));
require_once PFE_BASE . '/SQLfuncs.php';
require_once PFE_BASE . '/Layoutfuncs.php';
require_once PFE_BASE . '/Miscfuncs.php';
require_once PFE_BASE . '/Util.php';
require_once PFE_BASE . '/Validate.php';

if (!CheckConfig()) {
    require PFE_TEMPLATES . '/common-header.inc';
    PrintMissingConfig();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

if (!Util::GetPost('network_object') || !Util::GetPost('period_start') || !Util::GetPost('period_end')
    || !Util::GetPost('order') || !Util::GetPost('RepType')) { 
    HandleReportFormError();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

/* assumed that we have all parameters, let's do some brief check */
$fail = FALSE;
if (!is_numeric(Util::getFormData('RepType'))) $fail = TRUE;
if (!is_numeric(Util::getFormData('limit'))) $fail = TRUE;
if (ValidateDateFormat(urldecode(Util::getFormData('period_start'))) < 0) $fail = TRUE;
if (ValidateDateFormat(urldecode(Util::getFormData('period_end'))) < 0) $fail = TRUE;
if (ValidateObsPoint(urldecode(Util::getFormData('ObsPoint'))) < 0) $fail = TRUE;
if (ValidateCapability(urldecode(Util::getFormData('network_object')), Auth::getAuth(), urldecode(Util::getFormData('ObsPoint'))) < 0) $fail = TRUE;

if ($fail) {
    HandleReportFormError();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

/* the user has selected the 'start'-period and 'end'-period points; we here
   use them to produce the results. We don't need the period point breakdown,
   this is because we set to FALSE the 'select_si' value */
$Period = array();
$Period['start'] = Util::GetPost('period_start');
$Period['end'] = Util::GetPost('period_end');
$Period['select_si'] = FALSE; 

$SelectedObsPoint = SelectObsPoint(Util::GetPost('ObsPoint'));
if (!isset($SelectedObsPoint['id'])) {
    PrintNoReportEntries(); 
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$SelectedReportType = SelectReportType(Util::GetPost('RepType'));
if (!isset($SelectedReportType['id'])) {
    PrintNoReportForm();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

$ReportTableEntries = array();
GetTrafficReportEntries($SelectedObsPoint, $SelectedReportType, Util::GetPost('network_object'), FALSE, $Period,
			Util::GetPost('order'), Util::GetPost('limit'), $ReportTableEntries);

if (!isset($ReportTableEntries[0])) {
    PrintNoReportEntries();
    require PFE_TEMPLATES . '/footer.inc';
    exit();
}

PrintTrafficReport($SelectedObsPoint, $SelectedReportType, $Period, Util::GetPost('network_object'), $ReportTableEntries);

require PFE_TEMPLATES . '/footer.inc';

