<?php
/*
    pmacct-fe is Copyright (C) 2005 by Paolo Lucente

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

function HandleBytes($Field, $decimal = 2)
{
    if ($Field >= 1000000000) {
        $Field = bcdiv($Field, 1000000000, $decimal);
	$result = $Field . " Gb";
        return $result; 
    }
    else if ($Field >= 1000000) {
        $Field = bcdiv($Field, 1000000, $decimal);
	$result = $Field . " Mb";
	return $result;
    }
    else if ($Field >= 1000) {
        $Field = bcdiv($Field, 1000, $decimal > 1 ? $decimal-1 : $decimal);
	$result = $Field . " Kb";
	return $result;
    }
    else { 
        $result = $Field . " b";
	return $result;
    }
}

function HandleBytesPerSecond($Field, $Start, $End)
{
    $BitConv = 8;
    $decimal = 2;

    $Stime = strtotime($Start);
    $Etime = strtotime($End);
    $Delta = $Etime - $Stime;

    $Rate = bcdiv($Field, $Delta, 0);
    $Rate = $Rate * $BitConv;

    if ($Rate >= 1000000000) {
        $Rate = bcdiv($Rate, 1000000000, $decimal);
        $Rate = $Rate . " Gbit/s";
        return $Rate;
    }
    else if ($Rate >= 1000000) {
        $Rate = bcdiv($Rate, 1000000, $decimal);
        $Rate = $Rate . " Mbit/s";
        return $Rate;
    }
    else if ($Rate >= 1000) {
        $Rate = bcdiv($Rate, 1000, $decimal-1);
        $Rate = $Rate . " Kbit/s";
        return $Rate;
    }
    else {
        $Rate = $Rate . " bit/s";
        return $Rate;
    }
}

function HandlePktsPerSecond($Field, $Start, $End)
{
    $decimal = 2; 

    $Stime = strtotime($Start);
    $Etime = strtotime($End);
    $Delta = $Etime - $Stime;

    $Rate = bcdiv($Field, $Delta, $decimal);

    if ($Rate >= 1000000000) {
        $Rate = bcdiv($Rate, 1000000000, $decimal);
        $Rate = $Rate . "*10&sup9 pkt/s";
        return $Rate;
    }
    else if ($Rate >= 1000000) {
        $Rate = bcdiv($Rate, 1000000, $decimal);
        $Rate = $Rate . "*10&sup6 pkt/s";
        return $Rate;
    }
    else if ($Rate >= 1000) {
        $Rate = bcdiv($Rate, 1000, $decimal);
        $Rate = $Rate . "*10&sup3 pkt/s";
        return $Rate;
    }
    else {
        $Rate = $Rate . " pkt/s";
        return $Rate;
    }
}

function CacheResults($varname, &$payload)
{
    $_SESSION[$varname] = $payload;
}

function GetCachedResults($varname, &$payload)
{
    if (isset($_SESSION[$varname])) $payload = $_SESSION[$varname]; 
}

function DestroyCachedResults($object = FALSE)
{
    if (!$object) {
        if (isset($_SESSION['obs_points'])) unset($_SESSION['obs_points']);
        if (isset($_SESSION['reports'])) unset($_SESSION['reports']);
        if (isset($_SESSION['params'])) unset($_SESSION['params']);
	if (isset($_SESSION['config'])) unset($_SESSION['config']);
	if (isset($_SESSION['defaults'])) unset($_SESSION['defaults']);
    }
    else {
	if (isset($_SESSION[$object])) unset($_SESSION[$object]);
    }
}

function SelectReportType($Type)
{
    foreach ($_SESSION['reports'] as $row) { 
	if ($Type == $row['id']) return $row;
    }

    return;
}

function CheckConfig() 
{
    if (!isset($_SESSION['config'])) {
	return FALSE;
    }

    return TRUE;
}

function FindBasetime($now = FALSE, &$astep)
{
    if (!$now) $now = time(); 
    $ndate = localtime($now, 1);
    $rdate = RoundOff_Time($ndate, $astep['unit']);   
    $rtime = mktime($rdate['tm_hour'], $rdate['tm_min'], 0, $rdate['tm_mon']+1, $rdate['tm_mday'], $rdate['tm_year']+1900, $rdate['tm_isdst']);
    while ($rtime+$astep['secs'] <= $now) $rtime += $astep['secs']; 

    return $rtime;
}

function SplitStep($step)
{
    $astep = array();

    $pattern = "/^[\d]+/";
    preg_match($pattern, $step, $match);
    if (isset($match[0])) {
	$astep['num'] = $match[0];
	unset($match[0]);
    }

    $pattern = "/[\w]$/"; 
    preg_match($pattern, $step, $match);
    if (isset($match[0])) $astep['unit'] = $match[0];

    return $astep;
}

function CalcStep(&$step)
{
    switch($step['unit']) {
    case 'm': /* minutes */
        $step['secs'] = $step['num'] * 60;
        break;
    case 'h': /* hours */
        $step['secs'] = $step['num'] * 3600;
        break;
    case 'd': /* day */
        $step['secs'] = $step['num'] * 86400;
        break;
    default: /* invalid */
	exit();
    }
}

function RoundOff_Time(&$time, $unit)
{
    $local = $time;

    switch($unit) {
    case 'm': /* minutes */
	$local['tm_min'] = 0;
	break;
    case 'h': /* hours */
	$local['tm_hour'] = 0;
	$local['tm_min'] = 0;
	break;
    case 'd': /* day */
	$local['tm_mday'] = 1;
        $local['tm_hour'] = 0;
        $local['tm_min'] = 0;
        break;
    default: /* invalid */
	exit();
    }

    return $local;
}

function CalcPrevBasetimes(&$Points, &$step, $NumPoints)
{
    $index = $NumPoints-2;

    while ($index >= 0) {
        $Points[$index]['time'] = $Points[$index+1]['time'] - $step['secs'];
	$index--;
    }
}

function CalcPrevDates(&$Points, $NumPoints)
{
    $index = 0;

    while ($index < $NumPoints) {
	$Points[$index]['date'] = date("Y-m-d H:i:s", $Points[$index]['time']);
	$index++;
    }
}

function MatchEntries(&$Points, $Entries, $NumPoints)
{
    $index = 0;

    while ($index < $NumPoints) {
	foreach($Entries as $key => $row) {
	    if ($Points[$index]['date'] == $row['stamp_inserted']) {
		$Points[$index]['tot_bytes'] = $row['tot_bytes'];
		unset($Entries[$key]);
		break;
	    }
	}
	if (!isset($Points[$index]['tot_bytes'])) $Points[$index]['tot_bytes'] = 0;
	$index++;
    }
}

function getPeriodSI(&$Period)
{
    if ($Period['start'] && $Period['end'] && !$Period['select_si']) return FALSE;
    else return TRUE;
}

function getPeriod(&$Period)
{
    if (!$Period['start'] && !$Period['end']) return FALSE;
    else return TRUE; 
}

