<?php
/*
    pmacct-fe is Copyright (C) 2005 by Paolo Lucente

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

@define('PFE_BASE', dirname(__FILE__));
@define('PFE_TEMPLATES', dirname(__FILE__) . '/templates');
@define('HORDE_BASE', dirname(__FILE__) . '/..');
require_once HORDE_BASE . '/lib/base.php';

$pfe_configured = (@is_readable(PFE_BASE . '/config/conf.php'));

if (!$pfe_configured) {
    require PFE_BASE . '/../lib/Test.php';
    Horde_Test::configFilesMissing('PFE', PFE_BASE, array('conf.php'));
}
else {
    require_once PFE_BASE . '/config/conf.php';
}

/* User has not logged in before; just go to the login page. */
if (!Auth::isAuthenticated()) {
    Horde::authenticationFailureRedirect();
}
else {
    $pfe_user = Auth::getAuth();
}

$title = $config['prefs']['apptitle'] . ' :: ' . $config['prefs']['title'];
require PFE_TEMPLATES . '/common-header.inc';
require PFE_TEMPLATES . '/menu.inc';
$notification->notify(array('listeners' => 'status'));
require_once PFE_BASE . '/SQLfuncs.php';
require_once PFE_BASE . '/Layoutfuncs.php';
require_once PFE_BASE . '/Miscfuncs.php';
require_once PFE_BASE . '/Util.php';
require_once PFE_BASE . '/Validate.php';

/* Defaulting to pmacct */
unset($SelectedObsPoint);
unset($SelectedReportType);
unset($ObsPoints);
unset($ReportDescriptions);
unset($Capabilities);

if (Util::GetPost('Reload') || (!Util::GetPost('ObsPoint') && !Util::GetPost('RepType'))) {
    DestroyCachedResults();
    CacheResults('config', $config);
    CacheResults('defaults', $Defaults);
}
else {
    DestroyCachedResults('config');
    DestroyCachedResults('defaults');
    CacheResults('config', $config);
    CacheResults('defaults', $Defaults);
}
if (Util::GetPost('ObsPoint')) $SelectedObsPoint = Util::GetPost('ObsPoint');
if (Util::GetPost('RepType')) $SelectedReportType = Util::GetPost('RepType');

$ObsPoints = array();
GetCachedResults('obs_points', $ObsPoints);
if (!isset($ObsPoints[0])) { 
    GetObsPoints($ObsPoints);
    CacheResults('obs_points', $ObsPoints);
}

$ReportDescriptions = array();
GetCachedResults('reports', $ReportDescriptions);
if (!isset($ReportDescriptions[0])) {
    GetReportDescriptions($ReportDescriptions);
    CacheResults('reports', $ReportDescriptions);
}

if (isset($SelectedObsPoint)) { /* The user has already selected an Observation point */
    foreach ($ObsPoints as $row) {
      if ($SelectedObsPoint == trim($row['id'])) {
        $SelectedObsPoint = $row;
	break;
      }
    }

    $Capabilities = array();
    GetCapabilities($pfe_user, $SelectedObsPoint, $Capabilities);
    if (!isset($Capabilities[0])) { /* No capabilities on the selected Observation Point */
        PrintNoCapabilities();
        require PFE_TEMPLATES . '/footer.inc';
        exit();
    }

    if (isset($SelectedReportType)) { /* The user has already selected a Report Type */

	$Report = SelectReportType($SelectedReportType); 
	if (isset($Report['report_prefs'])) $ReportPrefs = $Report['report_prefs'];
	isset($ReportPrefs['type']) ? $FinalReportType = $ReportPrefs['type'] : $FinalReportType = $SelectedReportType;

        switch($FinalReportType) {
	case 1: /* Traffic Report */
	     if (ValidateReportNo1($ReportPrefs, $SelectedReportType) < 0) {
        	PrintObsPointSelectionBox();
       		ReportTypeSelectionBox();
        	PrintReportValidationError();
        	PrintReloadPageBox("Reload page");
		break;
	     }
             $Dates = array();
             GetDates($SelectedObsPoint, isset($ReportPrefs['table_in']) ? $ReportPrefs['table_in'] : $Defaults['table_in'], $Dates);

	     PrintObsPointSelectionBox();
	     ReportTypeSelectionBox();
             PrintTrafficReportForm($SelectedObsPoint, $Report, $Capabilities, $Dates);
	     PrintReloadPageBox("Reload page");

	     break;
	/* XXX: Insert new report types here */ 
	default:
             PrintObsPointSelectionBox();
             ReportTypeSelectionBox();
	     PrintNoReportForm();
	     PrintReloadPageBox("Reload page");
	     break;
	}
    }
    else {
        PrintObsPointSelectionBox();
        ReportTypeSelectionBox();
	PrintReloadPageBox("Reload page");
    }
}
else if (isset($ObsPoints[0])) { /* Observation point not selected; we offer available observation points */
    PrintObsPointSelectionBox();
    PrintReloadPageBox("Reload page");
}
else { /* No observation points available */
    PrintNoObsPoints();
}


require PFE_TEMPLATES . '/footer.inc';

