--
-- # su - postgres  (or whatever your database runs as ... usually postgres)
-- $ psql -d pmacct -f pmacct-create-table.pgsql
--

CREATE TABLE acct_in (
    ip_dst inet NOT NULL,
    packets integer NOT NULL,
    bytes bigint NOT NULL,
    stamp_inserted timestamp without time zone DEFAULT ('now'::text)::timestamp(0) with time zone NOT NULL,
    stamp_updated timestamp without time zone
);

CREATE TABLE acct_out (
    ip_src inet DEFAULT '0.0.0.0' NOT NULL,
    packets integer NOT NULL,
    bytes bigint NOT NULL,
    stamp_inserted timestamp without time zone DEFAULT ('now'::text)::timestamp(0) with time zone NOT NULL,
    stamp_updated timestamp without time zone
);

CREATE TABLE srchost_dstport (
    ip_src inet DEFAULT '0.0.0.0' NOT NULL,
    port_dst integer DEFAULT 0 NOT NULL,
    ip_proto smallint DEFAULT 0 NOT NULL,
    packets integer NOT NULL,
    bytes bigint NOT NULL,
    stamp_inserted timestamp without time zone DEFAULT ('now'::text)::timestamp(0) with time zone NOT NULL,
    stamp_updated timestamp without time zone
);

CREATE TABLE dsthost_srcport (
    ip_dst inet DEFAULT '0.0.0.0' NOT NULL,
    port_src integer DEFAULT 0 NOT NULL,
    ip_proto smallint DEFAULT 0 NOT NULL,
    packets integer NOT NULL,
    bytes bigint NOT NULL,
    stamp_inserted timestamp without time zone DEFAULT ('now'::text)::timestamp(0) with time zone NOT NULL,
    stamp_updated timestamp without time zone
);

CREATE TABLE dsthost_dstport (
    ip_dst inet DEFAULT '0.0.0.0' NOT NULL,
    port_dst integer DEFAULT 0 NOT NULL,
    ip_proto smallint DEFAULT 0 NOT NULL,
    packets integer NOT NULL,
    bytes bigint NOT NULL,
    stamp_inserted timestamp without time zone DEFAULT ('now'::text)::timestamp(0) with time zone NOT NULL,
    stamp_updated timestamp without time zone
);

CREATE TABLE srchost_srcport (
    ip_src inet DEFAULT '0.0.0.0' NOT NULL,
    port_src integer DEFAULT 0 NOT NULL,
    ip_proto smallint DEFAULT 0 NOT NULL,
    packets integer NOT NULL,
    bytes bigint NOT NULL,
    stamp_inserted timestamp without time zone DEFAULT ('now'::text)::timestamp(0) with time zone NOT NULL,
    stamp_updated timestamp without time zone
);


GRANT INSERT,SELECT,UPDATE,DELETE ON TABLE acct_in TO pmacct;
GRANT INSERT,SELECT,UPDATE,DELETE ON TABLE acct_out TO pmacct;
GRANT INSERT,SELECT,UPDATE,DELETE ON TABLE srchost_dstport TO pmacct;
GRANT INSERT,SELECT,UPDATE,DELETE ON TABLE dsthost_srcport TO pmacct;
GRANT INSERT,SELECT,UPDATE,DELETE ON TABLE dsthost_dstport TO pmacct;
GRANT INSERT,SELECT,UPDATE,DELETE ON TABLE srchost_srcport TO pmacct;

CREATE INDEX dd_si_idx ON dsthost_dstport USING btree (stamp_inserted);
CREATE INDEX sd_si_idx ON srchost_dstport USING btree (stamp_inserted);
CREATE INDEX ss_si_idx ON srchost_srcport USING btree (stamp_inserted);
CREATE INDEX ds_si_idx ON dsthost_srcport USING btree (stamp_inserted);
CREATE INDEX ao_si_idx ON acct_out USING btree (stamp_inserted);

ALTER TABLE ONLY srchost_dstport
    ADD CONSTRAINT srchost_dstport_pk PRIMARY KEY (ip_src, port_dst, ip_proto, stamp_inserted);

ALTER TABLE ONLY dsthost_srcport
    ADD CONSTRAINT dsthost_srcport_pk PRIMARY KEY (ip_dst, port_src, ip_proto, stamp_inserted);

ALTER TABLE ONLY dsthost_dstport
    ADD CONSTRAINT dsthost_dstport_pk PRIMARY KEY (ip_dst, port_dst, ip_proto, stamp_inserted);

ALTER TABLE ONLY srchost_srcport
    ADD CONSTRAINT srchost_srcport_pk PRIMARY KEY (ip_src, port_src, ip_proto, stamp_inserted);

ALTER TABLE ONLY acct_in
    ADD CONSTRAINT acct_in_pk PRIMARY KEY (ip_dst, stamp_inserted);

ALTER TABLE ONLY acct_out
    ADD CONSTRAINT acct_out_pk PRIMARY KEY (ip_src, stamp_inserted);

