<?php
/*
 *      Check REPORT-TYPES document to learn the structure expected by each existing Report Type.
 *
 *	Let's start from a basic SQL query: 
 *      'SELECT <sfield> FROM <table> WHERE <qfield> = <qvalue>'
 *
 *      <qvalue> is taken from the 'object' field of the 'capabilities' table ('pmacct_fe' DB). It's the
 *               set of objects the user can select from the specified Observation Point. 
 *
 *      <qfield> is the SQL field whose values are compared (searching for matches) against the <qvalue>;
 *               The <qfield> is taken from the <table> ('pmacct' DB). 
 *
 *      <sfield> is the SQL field whose values are presented to the user. It's also taken from the <table>
 *		 ('pmacct' DB).
 *
 *      no_ports = Port tables are not defined or not needed. 
 * 
 *      enable_grouping = The <table> sports unused fields other than the <sfield> one. Thus it's needed
 *		 	  to append to the query a 'GROUP' clause.  
 *
 *	stepping = It has to change accordingly to the 'sql_history' value. It's mandatory for graphing
 *                 purposes. Otherwise it's ignored. 
 *
 *	type = Points to a defined Report Type. For a list of defined RTs, check the REPORT-TYPES document. 
 *                        
 *      
 */

/*
 *	The Preferences Template below shows how to use the Report Type 1 in conjunction with primitives
 *	to which the concept of direction doesn't apply (eg. traffic breakdown by either VLAN ID, Transport
 *	layer protocol, ToS/CoS field and sometimes Agent ID). While this Preferences Template fits well
 *	with Sum types, they should not be preferred to the Incoming vs Outcoming way.
 *
 *	By assigning the same value to each of the couples table_in/table_out, qfield_in/qfield_out and
 * 	sfield_in/sfield_out, pmacct-fe is instructed to not operate in the Incoming vs Outcoming way (as
 *	it does with PT 101 and 102 instead), modifying the layout too. 
 */

$report_prefs = array(
    'type' => '1',		/* Mandatory ! */
    'no_ports' => '0',
    'table_in' => 'acct_v3',
    'table_out' => 'acct_v3',
    'qfield_in' => 'vlan',
    'qfield_out' => 'vlan',
    'sfield_in' => 'vlan',
    'sfield_out' => 'vlan',
    'sfield_label' => 'VLAN id',
    'enable_grouping' => '0',
    'stepping' => '1h',
);

