#!/bin/bash
#
#   pmacct-fe is Copyright (C) 2005 by Paolo Lucente
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#

function main_dir()
{
	filelist=`ls`
	for f in $filelist; do preserving_copy $f $1/$f 0; done
}

function admin_dir()
{
	# We check the files existence because older versions of the
	# package contain an empty 'admin' tree. Just a cosmetic check. 
        if [ `ls admin/ | wc -l` -gt 0 ] 
	then
	  dir="admin"
          test_dir $1/$dir/
          check_wx_dir $1/$dir/
	  filelist=`ls $dir/`
	  for f in $filelist; do preserving_copy $dir/$f $1/$dir/$f 0; done
	fi
}

function docs_dir()
{
	dir="docs"
        test_dir $1/$dir/
        check_wx_dir $1/$dir/
	filelist=`ls $dir/`
	for f in $filelist; do preserving_copy $dir/$f $1/$dir/$f 0; done
}

function scripts_dir()
{
	dir="scripts"
        test_dir $1/$dir/
        check_wx_dir $1/$dir/
	filelist=`ls $dir`
	for f in $filelist; do preserving_copy $dir/$f $1/$dir/$f 0; done 

	dir="scripts/housekeeping"
        test_dir $1/$dir/
        check_wx_dir $1/$dir/
	filelist=`ls $dir`
	for f in $filelist; do preserving_copy $dir/$f $1/$dir/$f 1; done
}

function templates_dir()
{
	dir="templates"	
        test_dir $1/$dir/
        check_wx_dir $1/$dir/
        filelist=`ls $dir`
        for f in $filelist; do preserving_copy $dir/$f $1/$dir/$f 0; done

        dir="templates/reports"
        test_dir $1/$dir/
        check_wx_dir $1/$dir/
        filelist=`ls $dir`
        for f in $filelist; do preserving_copy $dir/$f $1/$dir/$f 1; done
}

#
# [MANDATORY]  $1 == from
# [MANDATORY]  $2 == to 
# [MANDATORY]  $3 == preserve if exists
# [OPTIONAL]   $4 == preserving suffix
#
function preserving_copy()
{
	if [ -d $1 -o -h $1 ]
	then
	  return
	fi

	if [ $3 == 1 -a -e $2 ]
	then
	  echo "[PRESERVED] $2"
	  echo "[WRITTEN] $2.new" 
	  cp -f $1 $2.new
	  return
	fi

	echo "[UPGRADED] $2"
	cp -f $1 $2
} 

function test_dir()
{
	if [ ! -d $1 ]
	then
	  echo
	  echo "[ERROR] $1 is not a valid directory."
	  echo
	  exit 3
	fi
}

function check_wx_dir()
{
	if [ ! -w $1 -o ! -x $1 ]
	then
  	  echo
  	  echo "[ERROR] $1 is either not writable or executable for user '$myuid'."
  	  echo
  	  exit 4
	fi
}



myuid=`whoami`

if [ -z $1 ]
then
  echo
  echo "[USAGE] $0 <installation path>"
  echo
  exit 2
fi

test_dir $1
check_wx_dir $1

# Setting current and old versions 
CURRENT_VERSION=`cat VERSION`
if [ -r $1/VERSION ] 
then
  OLD_VERSION=`cat $1/VERSION`
else
  OLD_VERSION="0.1"
fi

echo
echo "OLD version:	$OLD_VERSION"
echo "NEW version:	$CURRENT_VERSION"  
echo "[ADVICE] Are you sure you wish to proceed with the upgrade in '$1' ?"
echo "		To abort the process press [CTRL+C]; to continue press [ENTER]"
echo

read reply 

main_dir $1
admin_dir $1
docs_dir $1
scripts_dir $1
templates_dir $1
